@@ -61,6 +61,7 @@ an object NDB will call the method with "del" argument.
 API
 ===
 '''
+
 import collections
 import errno
 import json
@@ -123,6 +124,7 @@ class RTNL_Object(dict):
     key_extra_fields = []
     hidden_fields = []
     fields_cmp = {}
+    fields_load_transform = {}
     field_filter = object
     rollback_chain = []
 
@@ -300,6 +302,9 @@ class RTNL_Object(dict):
         self.knorm = self.schema.compiled[self.table]['norm_idx']
         self.spec = self.schema.compiled[self.table]['all_names']
         self.names = self.schema.compiled[self.table]['norm_names']
+        self.lookup_fallbacks = self.schema.compiled[self.table][
+            'lookup_fallbacks'
+        ]
         self.names_count = [self.names.count(x) for x in self.names]
         self.last_save = None
         if self.event_map is None:
@@ -1078,9 +1083,10 @@ class RTNL_Object(dict):
         Load a value and clean up the `self.changed` set if the
         loaded value matches the expectation.
         '''
+        if key in self.fields_load_transform:
+            value = self.fields_load_transform[key](value)
         if self.load_debug:
             self.log.debug('load %s: %s' % (key, value))
-
         if key not in self.changed:
             dict.__setitem__(self, key, value)
         elif self.get(key) == value:
